/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     | Version:     4.1
    \\  /    A nd           | Web:         http://www.foam-extend.org
     \\/     M anipulation  | For copyright notice see file Copyright
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Application
    scalarTransportFoam

Description
    Solves a transport equation for a passive scalar

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "simpleControl.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //


int main(int argc, char *argv[])
{
#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"

    simpleControl simple(mesh);

#   include "createFields.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nCalculating scalar transport\n" << endl;

#   include "CourantNo.H"

    while (simple.loop())
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;
	
	// Allocated memory does not get freed
	scalarField* T_testptr = new scalarField(T.size(), 0);
	Info << *T_testptr << endl;
	 delete T_testptr; // solution
	
	// Dynamic allocated array
	// scalar *p = new scalar[5](5,4,3,2,1);
	// delete [p] p; // solution for dynamically allocated array
	
	// Uninitialized variables
	// scalar volume;
	// scalar volume{0}; // solution
	// Info << volume << endl;
	// if (volume > 0)
	// 	Info << "I am higher than 0" << endl;
	// else if (volume == 0)
	// 	Info << "I am 0" << endl;
	
	// Segmentation fault
	// - Out of bounds
	// label* labelPtr = new label[4](4,3,2,1);
	// Info << labelPtr[0] << endl;
	// labelPtr[5] = 12; // out of bounds
	// delete[] labelPtr;
	
	// - Dereference of a null pointer
	// label* labelPtr = NULL;
	// *labelPtr = 2;
       
        // - Using after freeing the memory 	
	// label* labelPtr = new label[4](4,3,2,1);
	// Info << labelPtr[0] << endl;
	// delete[] labelPtr;
	// labelPtr[0] = 12; // out of bounds
	while (simple.correctNonOrthogonal())
        {
            solve
            (
                fvm::ddt(T)
              + fvm::div(phi, T)
              - fvm::laplacian(DT, T)
            );
        }

        runTime.write();
    }

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
